##############################################

Programmierpraktikum: Datensysteme WISE 24/25

##############################################

Quick Start Guide to build the project and run all tests:
----------------------------------------------------------
* 0) Unittests MIT Benchmark
  - cd <project>/src
  - make unittest
  - ./unit_test

* 1) Standard MIT Benchmark
  - cd <project>/src
  - make harness_test
  - ./speed_test 1468 0 0 0 0 4000 16000

* 2) All the tests from above
  - cd <project>/src
  - make all 
  - ./unit_test
  - ./speed_test 1468 0 0 0 0 4000 16000

* 3) Compile and run all the tests
  - cd <project>/src
  - make test ./speed_test 1468 0 0 0 0 4000 16000

* 4) Run persistence tests:
  - cd <project>/src
  - make insert_test 
  - make get_test
  - ./run_persistence_test.sh

Goal:
------------------
  - Create your own index structure (BTree) and implement multithreaded transaction handling and a persistence mechanism 
  - Implement the API provided in server.h
    - Every function header contains a detailed description of the expected functionality.
  - Additional information for this assignment can be found in  Elizabeth G. Reid's master thesis [1]
    Elizabeth G. Reid, Design and Evaluation of a Benchmark for Main Memory Transaction Processing Systems
    and in the PPDS lecture slides [2]

Test your implementation:
------------------
  - In order to pass this course the ```unittest```, ```speed_test```, ```insert_test```, and ```get_test``` that can be found in the ```driver``` directory have to pass. 
  - Additionally, the ```run_persitence_test.sh``` has to pass. 

  - Write additional unit tests where you also test some special use cases 
    (HINT: Take a look at [1])
- Running the performance test ```./speed_test 1468 0 0 0 0 4000 16000``` on a correct 
  implementation you should receive an output similar to this where the individual execution time may differ depending on your implementation:

  ````
  speed_test called with 4000 populate inserts per thread and 16000 tests per thread

  Running the Speed Test, seed = 1468
  Creating 29 indices.
  Populating indices 29.
  Time to populate: 37 milliseconds.
  Testing the indices.
  Time to test: 1337 milliseconds.
  Testing complete.
          NUM_DEADLOCK: 0
          NUM_TXN_FAIL: 0
          NUM_TXN_COMP: 800000
  Overall time to run: 1375 milliseconds.

  Cleaning up.
  ````

  - The minimum performance target for your implementation is: 20000 transactions per second

System specifications:
-----------------
We will test your implementation on our scale-up box:
  - CPU: Intel(R) Xeon(R) Gold 6338 CPU @ 2.00GHz
  - Cores: 128
  - RAM: 1008GB
  - OS: Ubuntu 20.04
  - Compiler: gcc-11

Make sure that your implementation compiles with gcc-11 and runs on Ubuntu 20.04! 


----------------
[1] Elizabeth G. Reid, Design and Evaluation of a Benchmark for Main Memory Transaction Processing Systems, 
[Link](https://core.ac.uk/download/pdf/4417736.pdf)

[2] Programmierpraktikum: Datensysteme WiSe2024/25, 
[Link](https://mboehm7.github.io/teaching/ws2425_ppds/index.htm)